<?php

namespace Tests\Unit\Controllers;

use App\Rules\StrLowercase;
use App\Rules\URL\KeywordBlacklist;
use App\Url;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Tests\TestCase;

/**
 * @coversDefaultClass App\Http\Controllers\UrlController
 */
class UrlControllerTest extends TestCase
{
    /**
     * The user_id column in the table of Url must be null.
     *
     * @test
     * @group u-controller
     * @covers ::create
     */
    public function shortenUrl_user_id()
    {
        $long_url = 'https://laravel.com';

        $this->post(route('createshortlink'), [
            'long_url' => $long_url,
        ]);

        $url = Url::whereLongUrl($long_url)->first();

        $this->assertSame(null, $url->user_id);
    }

    /**
     * The user_id column in the table of Url must be filled in with the value
     * of the authenticated user id.
     *
     * @test
     * @group u-controller
     * @covers ::create
     */
    public function shortenUrl_user_id_2()
    {
        $user = $this->admin();
        $long_url = 'https://laravel.com';

        $this->loginAsAdmin();

        $this->post(route('createshortlink'), [
            'long_url' => $long_url,
        ]);

        $url = Url::whereLongUrl($long_url)->first();

        $this->assertSame($user->id, $url->user_id);
    }

    /**
     * The keyword column in the table of Url must be filled with a value that
     * generated by UrlHub. The is_custom column must be filled with 0 / false.
     *
     * @test
     * @group u-controller
     * @covers ::create
     */
    public function shortenUrl_not_custom()
    {
        $long_url = 'https://laravel.com';

        $this->post(route('createshortlink'), [
            'long_url' => $long_url,
        ]);

        $url = Url::whereLongUrl($long_url)->first();

        $this->assertEquals(
            config('urlhub.hash_length'),
            strlen($url->keyword)
        );
        $this->assertFalse($url->is_custom);
    }

    /**
     * The keyword column in the table of Url must be filled in with the value
     * given by User. The is_custom column must be filled with 1 / true.
     *
     * @test
     * @group u-controller
     * @covers ::create
     */
    public function shortenUrl_is_custom()
    {
        config()->set('urlhub.hash_length', 6);

        $long_url = 'https://laravel.com';
        $custom_url = 'laravel-http-tests';

        $this->post(route('createshortlink'), [
            'long_url'       => $long_url,
            'custom_keyword' => $custom_url,
        ]);

        $url = Url::whereLongUrl($long_url)->first();
        $this->assertSame($custom_url, $url->keyword);
        $this->assertTrue($url->is_custom);
    }

    /**
     * @test
     * @group u-controller
     * @covers ::checkExistingCustomUrl
     */
    public function check_existing_custom_url_pass()
    {
        $response = $this->post(route('home').'/custom-link-avail-check', [
            'keyword' => 'hello',
        ]);

        $response->assertJson(['success' => 'Available']);
    }

    /**
     * @test
     * @group u-controller
     * @covers ::checkExistingCustomUrl
     * @dataProvider checkExistingCustomUrl_fail
     */
    public function check_existing_custom_url_fail($data)
    {
        factory(Url::class)->create([
            'keyword' => 'laravel',
        ]);

        $request = new Request;

        $validator = Validator::make($request->all(), [
            'keyword' => [
                'max:20',
                'alpha_dash',
                'unique:urls',
                new StrLowercase,
                new KeywordBlacklist,
            ],
        ]);

        $response = $this->post(route('home').'/custom-link-avail-check', [
            'keyword' => $data,
        ]);

        $response->assertJson(['errors' => $validator->errors()->all()]);
    }

    public function checkExistingCustomUrl_fail()
    {
        return [
            [str_repeat('a', 50)],
            ['laravel~'],
            ['laravel'],
            ['Laravel'],
            ['login'],
        ];
    }
}
